@@ -1,11 +1,12 @@
 
 
+
 #' @title A function for creating internal documents
 #' @description Is used for creating xml documents which nearly every function of this package needs as an input
 #' @param bpmn_file file path of the BPMN file
 #' @param signavio boolean which indicates whether the file stems from signavio
 #' @return an object containing the xml document
-#' @examples 
+#' @examples
 #' \dontshow{file_path <- system.file("extdata", "doc.txt", package="understandBPMN")}
 #' create_internal_doc(file_path)
 #' @export
@@ -118,7 +119,8 @@ node_ids <-
     node_ids <- unlist(xmlApply(nodes, xmlGetAttr, name = "id"))
     if (sequential) {
       join_nodes <- join_gateways(xml_internal_doc, type, signavio)
-      split_nodes <- split_gateways(xml_internal_doc, type, signavio)
+      split_nodes <-
+        split_gateways(xml_internal_doc, type, signavio)
       connector_nodes <- unique(c(join_nodes, split_nodes))
       if (length(connector_nodes) != 0)
         node_ids <- node_ids[!(node_ids %in% connector_nodes)]
@@ -132,7 +134,7 @@ node_ids <-
 #' @param filter_non_connector_activities attribute indicating whether non connector activities should be filtered. The default value is FALSE.
 #' @param signavio boolean which indicates whether the file stems from signavio
 #' @return an object containing a table with the IDs and tasknames
-#' @examples 
+#' @examples
 #' \dontshow{file_path <- system.file("extdata", "doc.txt", package="understandBPMN")}
 #' task_names(create_internal_doc(file_path))
 #' @export
@@ -197,7 +199,8 @@ task_names <-
         unique(c(join_activities, split_activities))
     }
     task_id <- unlist(xmlApply(task_nodes, xmlGetAttr, name = "id"))
-    name_task_list <- xmlApply(task_nodes, xmlGetAttr, name = "name")
+    name_task_list <-
+      xmlApply(task_nodes, xmlGetAttr, name = "name")
     name_task_list <- lapply(name_task_list, function(name) {
       name[is.null(name)] <- " "
       return (name)
@@ -344,7 +347,8 @@ number_io_flows_gateway <-
             signavio = FALSE) {
     if (!signavio) {
       incoming <- number_incoming_flows(xml_internal_doc, gateway_type)
-      outgoing <- number_outgoing_flows(xml_internal_doc, gateway_type)
+      outgoing <-
+        number_outgoing_flows(xml_internal_doc, gateway_type)
     } else {
       incoming <-
         number_incoming_flows(xml_internal_doc, gateway_type, signavio)
@@ -574,3 +578,233 @@ all_join_gateways <- function(xml_internal_doc, signavio  = FALSE) {
   }
   return(joins)
 }
+
+#Returns the number of connected components for calculating the cyclomatic metric of McCabe
+number_connected_components <-
+  function(repetition_and_path_log,
+           xml_internal_doc,
+           signavio = FALSE) {
+    task_id <- NULL
+    task_ids <- task_names(xml_internal_doc, signavio = signavio)
+    if (length(task_ids) > 0)
+      task_ids <- task_ids %>% pull(task_id)
+    else
+      task_ids <- character()
+    task_ids <- as.character(task_ids)
+    if (!signavio) {
+      splits <-
+        split_gateways(
+          xml_internal_doc,
+          "//bpmn:exclusiveGateway | //exclusiveGateway | //bpmn:eventBasedGateway | //eventBasedGateway"
+        )
+      joins <-
+        join_gateways(
+          xml_internal_doc,
+          "//bpmn:exclusiveGateway | //exclusiveGateway | //bpmn:eventBasedGateway | //eventBasedGateway | //bpmn:task | //bpmn:sendTask | //bpmn:receiveTask |
+          //bpmn:manualTask | //bpmn:businessRuleTask | //bpmn:userTask | //bpmn:scriptTask |
+          //bpmn:subProcess | //bpmn:callActivity | //task"
+        )
+      event_nodes <-
+        getNodeSet(
+          xml_internal_doc,
+          "//bpmn:startEvent | //bpmn:messageStartEvent | //bpmn:timerStartEvent |
+          //bpmn:conditionalStartEvent | //bpmn:endEvent | //bpmn:messageEndEvent |
+          //bpmn:terminateEndEvent | //bpmn:escalationEndEvent | //bpmn:errorEndEvent |
+          //bpmn:compensationEndEvent | //bpmn:signalEndEvent | //bpmn:intermediateCatchEvent |
+          //bpmn:intermediateThrowEvent | //bpmn:boundaryEvent | //startEvent | //endEvent | //intermediateEvent"
+        )
+    } else {
+      splits <-
+        split_gateways(xml_internal_doc,
+                       "//xmlns:exclusiveGateway", signavio)
+      joins <-
+        join_gateways(
+          xml_internal_doc,
+          "//xmlns:exclusiveGateway | //xmlns:task | //xmlns:sendTask | //xmlns:receiveTask |
+          //xmlns:manualTask | //xmlns:businessRuleTask | //xmlns:userTask | //xmlns:scriptTask |
+          //xmlns:subProcess | //xmlns:callActivity | //xmlns:eventBasedGateway",
+          signavio
+        )
+      event_nodes <-
+        getNodeSet(
+          xml_internal_doc,
+          "//xmlns:startEvent | //xmlns:messageStartEvent | //xmlns:timerStartEvent |
+          //xmlns:conditionalStartEvent | //xmlns:endEvent | //xmlns:messageEndEvent |
+          //xmlns:terminateEndEvent | //xmlns:escalationEndEvent | //xmlns:errorEndEvent |
+          //xmlns:compensationEndEvent | //xmlns:signalEndEvent | //xmlns:intermediateCatchEvent |
+          //xmlns:intermediateThrowEvent | //xmlns:boundaryEvent",
+          namespace(xml_internal_doc)
+        )
+    }
+    XOR_loop_splits <-
+      lapply(repetition_and_path_log[[4]], function(path) {
+        indices_xor_loop_split <- which(path == "XOR-loop-split")
+        indices_xor_loop_split <- indices_xor_loop_split - 1
+        return(path[indices_xor_loop_split])
+      })
+    XOR_loop_splits <- unlist(XOR_loop_splits) %>% unique()
+    
+    XOR_loop_joins <-
+      lapply(repetition_and_path_log[[4]], function(path) {
+        indices_xor_loop_join <- which(path == "XOR-loop-join")
+        indices_xor_loop_join <- indices_xor_loop_join - 1
+        return(path[indices_xor_loop_join])
+      })
+    XOR_loop_joins <- unlist(XOR_loop_joins) %>% unique()
+    
+    joins <- setdiff(joins, XOR_loop_joins)
+    joins <- c(joins, XOR_loop_splits)
+    
+    splits <- setdiff(splits, XOR_loop_splits)
+    splits <- c(splits, XOR_loop_joins)
+    
+    nodes_connected_components <-
+      lapply(repetition_and_path_log[[4]], function(path) {
+        relevant_indices <-
+          which(path %in% c(task_ids, splits, joins, event_nodes))
+        path_after_filtering <- path[relevant_indices]
+        split_indices <- which(path_after_filtering %in% splits)
+        join_indices <- which(path_after_filtering %in% joins)
+        nesting_depth <- rep(0, length(path_after_filtering))
+        if (length(split_indices) > 0)
+          first_split_index <- split_indices[1]
+        else
+          first_split_index <- 1
+        
+        nesting_depth[split_indices] <- 1
+        join_indices <- join_indices[join_indices > first_split_index]
+        nesting_depth[join_indices] <- -1
+        nesting_depth <- cumsum(nesting_depth)
+        nesting_depth <-
+          ifelse(cumsum(nesting_depth) < 0, 0, cumsum(nesting_depth))
+        exclusive_split_gateways_level_zero <-
+          path_after_filtering[nesting_depth == 1 &
+                                 (path_after_filtering %in% splits)]
+        exclusive_level_zero_indices <-
+          which(path_after_filtering %in% exclusive_split_gateways_level_zero)
+        elements_before_exclusive_level_zero <-
+          path_after_filtering[exclusive_level_zero_indices - 1]
+        return(c(exclusive_split_gateways_level_zero))
+      })
+    nodes_connected_components <-
+      unlist(nodes_connected_components) %>% unique()
+    
+    return(length(nodes_connected_components))
+  }
+
+#Returns the number of tasks at depth level zero not before a split for calculating the cyclomatic metric of McCabe
+number_tasks_level_0_not_before_split <-
+  function(repetition_and_path_log,
+           xml_internal_doc,
+           signavio = FALSE) {
+    task_id <- NULL
+    task_ids <- task_names(xml_internal_doc, signavio = signavio)
+    if (length(task_ids) > 0)
+      task_ids <- task_ids %>% pull(task_id)
+    else
+      task_ids <- character()
+    task_ids <- as.character(task_ids)
+    if (!signavio) {
+      splits <-
+        split_gateways(
+          xml_internal_doc,
+          "//bpmn:exclusiveGateway | //exclusiveGateway | //bpmn:eventBasedGateway | //eventBasedGateway"
+        )
+      joins <-
+        join_gateways(
+          xml_internal_doc,
+          "//bpmn:exclusiveGateway | //exclusiveGateway | //bpmn:eventBasedGateway | //eventBasedGateway | //bpmn:task | //bpmn:sendTask | //bpmn:receiveTask |
+          //bpmn:manualTask | //bpmn:businessRuleTask | //bpmn:userTask | //bpmn:scriptTask |
+          //bpmn:subProcess | //bpmn:callActivity | //task"
+        )
+      event_nodes <-
+        getNodeSet(
+          xml_internal_doc,
+          "//bpmn:startEvent | //bpmn:messageStartEvent | //bpmn:timerStartEvent |
+          //bpmn:conditionalStartEvent | //bpmn:endEvent | //bpmn:messageEndEvent |
+          //bpmn:terminateEndEvent | //bpmn:escalationEndEvent | //bpmn:errorEndEvent |
+          //bpmn:compensationEndEvent | //bpmn:signalEndEvent | //bpmn:intermediateCatchEvent |
+          //bpmn:intermediateThrowEvent | //bpmn:boundaryEvent | //startEvent | //endEvent | //intermediateEvent"
+        )
+    } else {
+      splits <-
+        split_gateways(xml_internal_doc,
+                       "//xmlns:exclusiveGateway", signavio)
+      joins <-
+        join_gateways(
+          xml_internal_doc,
+          "//xmlns:exclusiveGateway | //xmlns:task | //xmlns:sendTask | //xmlns:receiveTask |
+          //xmlns:manualTask | //xmlns:businessRuleTask | //xmlns:userTask | //xmlns:scriptTask |
+          //xmlns:subProcess | //xmlns:callActivity | //xmlns:eventBasedGateway",
+          signavio
+        )
+      event_nodes <-
+        getNodeSet(
+          xml_internal_doc,
+          "//xmlns:startEvent | //xmlns:messageStartEvent | //xmlns:timerStartEvent |
+          //xmlns:conditionalStartEvent | //xmlns:endEvent | //xmlns:messageEndEvent |
+          //xmlns:terminateEndEvent | //xmlns:escalationEndEvent | //xmlns:errorEndEvent |
+          //xmlns:compensationEndEvent | //xmlns:signalEndEvent | //xmlns:intermediateCatchEvent |
+          //xmlns:intermediateThrowEvent | //xmlns:boundaryEvent",
+          namespace(xml_internal_doc)
+        )
+    }
+    XOR_loop_splits <-
+      lapply(repetition_and_path_log[[4]], function(path) {
+        indices_xor_loop_split <- which(path == "XOR-loop-split")
+        indices_xor_loop_split <- indices_xor_loop_split - 1
+        return(path[indices_xor_loop_split])
+      })
+    XOR_loop_splits <- unlist(XOR_loop_splits) %>% unique()
+    
+    XOR_loop_joins <-
+      lapply(repetition_and_path_log[[4]], function(path) {
+        indices_xor_loop_join <- which(path == "XOR-loop-join")
+        indices_xor_loop_join <- indices_xor_loop_join - 1
+        return(path[indices_xor_loop_join])
+      })
+    XOR_loop_joins <- unlist(XOR_loop_joins) %>% unique()
+    
+    joins <- setdiff(joins, XOR_loop_joins)
+    joins <- c(joins, XOR_loop_splits)
+    
+    splits <- setdiff(splits, XOR_loop_splits)
+    splits <- c(splits, XOR_loop_joins)
+    
+    tasks_level_0 <-
+      lapply(repetition_and_path_log[[1]], function(path) {
+        relevant_indices <-
+          which(path %in% c(task_ids, splits, joins, event_nodes))
+        path_after_filtering <- path[relevant_indices]
+        split_indices <- which(path_after_filtering %in% splits)
+        join_indices <- which(path_after_filtering %in% joins)
+        nesting_depth <- rep(0, length(path_after_filtering))
+        if (length(split_indices) > 0)
+          first_split_index <- split_indices[1]
+        else
+          first_split_index <- 1
+        
+        nesting_depth[split_indices] <- 1
+        join_indices <- join_indices[join_indices > first_split_index]
+        nesting_depth[join_indices] <- -1
+        nesting_depth <-
+          ifelse(cumsum(nesting_depth) < 0, 0, cumsum(nesting_depth))
+        exclusive_split_gateways_level_zero <-
+          path_after_filtering[nesting_depth == 1 &
+                                 (path_after_filtering %in% splits)]
+        tasks_level_0 <-
+          path_after_filtering[nesting_depth == 0 &
+                                 (path_after_filtering %in% task_ids)]
+        exclusive_level_zero_indices <-
+          which(path_after_filtering %in% exclusive_split_gateways_level_zero)
+        elements_before_exclusive_level_zero <-
+          path_after_filtering[exclusive_level_zero_indices - 1]
+        tasks_level_0_just_before_split <-
+          tasks_level_0[tasks_level_0 %in% elements_before_exclusive_level_zero]
+        tasks_level_0 <-
+          setdiff(tasks_level_0, tasks_level_0_just_before_split)
+      })
+    tasks_level_0 <- unlist(tasks_level_0) %>% unique()
+    
+    return(length(tasks_level_0))
+  }
